@@ -14,34 +14,10 @@
 ## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
-### Estimation for nested Archimedean copulas
+### Estimation for nested Archimedean copulas ##################################
 
 ### Auxiliaries ################################################################
 
-##' @title Pseudo-observations
-##' @param x matrix of random variates to be converted to pseudo-observations
-##' @param na.last passed to rank()
-##' @param ties.method passed to rank()
-##' @param lower.tail if FALSE, pseudo-observations when apply the empirical
-##'        marginal survival functions are returned.
-##' @return pseudo-observations (of the same dimensions as x)
-##' @author Marius Hofert & Martin Maechler
-pobs <- function(x, na.last = "keep",
-		 ## formals(rank) works in pre-2015-10-15 and newer version of rank():
-		 ties.method = eval(formals(rank)$ties.method),
-		 lower.tail = TRUE)
-{
-    ties.method <- match.arg(ties.method)
-    U <- if(!is.null(dim(x)))
-	     apply(x, 2, rank, na.last=na.last, ties.method=ties.method) / (nrow(x)+1)
-	 else
-	     rank(x, na.last=na.last, ties.method=ties.method) / (length(x)+1)
-    if(inherits(x, "zoo")) # incl "xts" (but no similar) -- FIXME? and use:
-### if(is.object(x) && !isS4(x) && !is.data.frame(x)) ## "zoo", "xts" et al
-	attributes(U) <- attributes(x)
-    if(lower.tail) U else 1-U
-}
-
 ##' @title Compute an initial interval or value for estimation procedures
 ##' @param family Archimedean family
 ##' @param tau.range vector containing lower and upper admissible Kendall's tau
