@@ -22,12 +22,12 @@
  *
  * @brief Tests of extreme-value dependence
  *
- *
  */
 
 #include <R.h>
 #include <Rmath.h>
 #include <R_ext/Applic.h>
+#include <R_ext/Utils.h>
 #include "copula.h"
 #include "empcop.h"
 
@@ -63,7 +63,7 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
   double *dert = Calloc(*p, double);
 
   double t, ecterm, process, mean, ind, indt, d, dt, denom,
-    invsqrtn = 1.0/sqrt(*n), tmpu, tmpv, diff, ecut, ecu;
+    invsqrtn = 1./sqrt(*n), tmpu, tmpv, diff, ecut, ecu;
 
   int i, j, k, l, c;
 
@@ -84,7 +84,7 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 	      vt[k] =  ut[k];
 	    }
 
-	  ecterm = R_pow(multCn(U, *n, *p, ut, 1, 0, 0.0), (1 - t)/t) / t;
+	  ecterm = R_pow(multCn(U, *n, *p, ut, 1, 0, 0.), (1 - t)/t) / t;
 
 	  /* derivatives */
 	  for (k = 0; k < *p; k++)
@@ -92,20 +92,20 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 	      /* bins = 2 invsqrtn */
 	      if (*der2n)
 		{
-		  denom = 2.0 * invsqrtn;
+		  denom = 2. * invsqrtn;
 
 		  /* u and v */
 		  if (u[k] < invsqrtn)
 		    {
 		      tmpu = u[k]; tmpv = v[k];
-		      u[k] = 2.0 * invsqrtn; v[k] = 0.0;
+		      u[k] = 2. * invsqrtn; v[k] = 0.;
 		      der[k] = der_multCn(U, *n, *p, u, v, denom);
 		      u[k] = tmpu; v[k] = tmpv;
 		    }
-		  else if (u[k] > 1.0 - invsqrtn)
+		  else if (u[k] > 1. - invsqrtn)
 		    {
 		      tmpu = u[k]; tmpv = v[k];
-		      u[k] = 1.0; v[k] = 1.0 - 2.0 * invsqrtn;
+		      u[k] = 1.; v[k] = 1. - 2. * invsqrtn;
 		      der[k] = der_multCn(U, *n, *p, u, v, denom);
 		      u[k] = tmpu; v[k] = tmpv;
 		    }
@@ -120,14 +120,14 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 		  if (ut[k] < invsqrtn)
 		    {
 		      tmpu = ut[k]; tmpv = vt[k];
-		      ut[k] = 2.0 * invsqrtn; vt[k] = 0.0;
+		      ut[k] = 2. * invsqrtn; vt[k] = 0.;
 		      dert[k] = der_multCn(U, *n, *p, ut, vt, denom);
 		      ut[k] = tmpu; vt[k] = tmpv;
 		    }
-		  else if (ut[k] > 1.0 - invsqrtn)
+		  else if (ut[k] > 1. - invsqrtn)
 		    {
 		      tmpu = ut[k]; tmpv = vt[k];
-		      ut[k] = 1.0; vt[k] = 1.0 - 2.0 * invsqrtn;
+		      ut[k] = 1.; vt[k] = 1. - 2. * invsqrtn;
 		      dert[k] = der_multCn(U, *n, *p, ut, vt, denom);
 		      ut[k] = tmpu; vt[k] = tmpv;
 		    }
@@ -141,12 +141,12 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 	      else /* bins decreasing near 0 and 1 */
 		{
 		  u[k] += invsqrtn; v[k] -= invsqrtn;
-		  denom = MIN(u[k], 1.0) - MAX(v[k], 0.0);
+		  denom = MIN(u[k], 1.) - MAX(v[k], 0.);
 		  der[k] = der_multCn(U, *n, *p, u, v, denom);
 		  u[k] -= invsqrtn; v[k] += invsqrtn;
 
 		  ut[k] += invsqrtn; vt[k] -= invsqrtn;
-		  denom = MIN(ut[k], 1.0) - MAX(vt[k], 0.0);
+		  denom = MIN(ut[k], 1.) - MAX(vt[k], 0.);
 		  dert[k] = der_multCn(U, *n, *p, ut, vt, denom);
 		  ut[k] -= invsqrtn; vt[k] += invsqrtn;
 		}
@@ -155,10 +155,10 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 	  /* for each pseudo-obs */
 	  for (i = 0; i < *n; i++)
 	    {
-	      ind = 1.0;
-	      indt = 1.0;
-	      d = 0.0;
-	      dt = 0.0;
+	      ind = 1.;
+	      indt = 1.;
+	      d = 0.;
+	      dt = 0.;
 	      for (k = 0; k < *p; k++)
 		{
 		  ind *= (U[i + k * (*n)] <= u[k]);
@@ -178,10 +178,10 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
   for (l = 0; l < *N; l++)
     {
       /* generate n variates */
-      mean = 0.0;
+      mean = 0.;
       for (i=0;i<*n;i++)
 	{
-	  random[i] = norm_rand(); /*(unif_rand() < 0.5) ? -1.0 : 1.0 ;*/
+	  random[i] = norm_rand(); /*(unif_rand() < 0.5) ? -1. : 1. ;*/
 	  mean += random[i];
 	}
       mean /= *n;
@@ -190,10 +190,10 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
       for (c = 0; c < *nt; c++)
 	{
 	  k = c + l * (*nt);
-	  s0[k] = 0.0;
+	  s0[k] = 0.;
 	  for (j = 0; j < *m; j++)
 	    {
-	      process = 0.0;
+	      process = 0.;
 	      for (i = 0; i < *n; i++)
 		process += (random[i] - mean) * influ[i + j * (*n) + c * (*n) * (*m)];
 
@@ -208,7 +208,7 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
   /* test statistic */
   for (c = 0; c < *nt; c++)
     {
-      stat[c] = 0.0;
+      stat[c] = 0.;
       t = tg[c];
       for (j = 0; j < *m; j++)
 	{
@@ -266,8 +266,8 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
   double *random = Calloc(*n, double);
 
   double pu, pv, sum, d1, d2,  process,
-    mean, invsqrtn = 1.0 / sqrt(*n), minTkSi,  minSkTi,
-    lb = 1.0 / (*n + 1.0), ub = *n / (*n + 1.0),
+    mean, invsqrtn = 1. / sqrt(*n), minTkSi,  minSkTi,
+    lb = 1. / (*n + 1.), ub = *n / (*n + 1.),
     A, cA0, cA1, t, loguv, loguvA, Aterm;
 
   int i, j, k, l;
@@ -286,13 +286,13 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
   /* correction terms */
   if (*CFG)
     {
-      cA0 = biv_logACFG(*n, S, T, 0.0);
-      cA1 = biv_logACFG(*n, S, T, 1.0);
+      cA0 = biv_logACFG(*n, S, T, 0.);
+      cA1 = biv_logACFG(*n, S, T, 1.);
     }
   else
     {
-      cA0 = biv_invAP(*n, S, T, 0.0);
-      cA1 = biv_invAP(*n, S, T, 1.0);
+      cA0 = biv_invAP(*n, S, T, 0.);
+      cA1 = biv_invAP(*n, S, T, 1.);
     }
 
   /* for each point of the grid */
@@ -306,27 +306,27 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
       d1 = der1bivCn(U, V, *n, u[j], v[j]);
       d2 = der2bivCn(U, V, *n, u[j], v[j]);
 
-      t = 1.0 / pv;
+      t = 1. / pv;
 
       if (*CFG)
 	{
 	  A = exp(biv_logACFG(*n, S, T, t)
-		  - (1.0 - t) * cA0 - t * cA1);
+		  - (1. - t) * cA0 - t * cA1);
 	  loguvA = loguv * A;
 	  Aterm = exp(loguvA) * loguvA;
 	}
       else
 	{
-	  A = 1.0 / (biv_invAP(*n, S, T, t)
-		     - (1.0 - t) * (cA0  - 1.0)
-		     - t * (cA1 - 1.0));
+	  A = 1. / (biv_invAP(*n, S, T, t)
+		     - (1. - t) * (cA0  - 1.)
+		     - t * (cA1 - 1.));
 	  loguvA = loguv * A;
 	  Aterm = exp(loguvA) * loguvA * A;
 	}
 
       for (i = 0; i < *n; i++) /* for each pseudo-obs */
 	{
-	  sum = 0.0;
+	  sum = 0.;
 	  for (k = 0; k < *n; k++)
 	    {
 	      minTkSi = MIN(pv * T[k], pu * S[i]);
@@ -340,7 +340,7 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
 		sum += MIN(pu * Sm[k], minTkSi) - MIN(pu * Sp[k], minTkSi)
 		  + MIN(pv * Tm[k], minSkTi) - MIN(pv * Tp[k], minSkTi);
 	    }
-	  sum *= invsqrtn / 2.0;
+	  sum *= invsqrtn / 2.;
 
 	  if (*CFG)
 	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
@@ -361,19 +361,19 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
   for (l = 0; l < *N; l++)
     {
       /* generate n variates */
-      mean = 0.0;
+      mean = 0.;
       for (i=0;i<*n;i++)
 	{
-	  random[i] = norm_rand(); /*(unif_rand() < 0.5) ? -1.0 : 1.0 ;*/
+	  random[i] = norm_rand(); /*(unif_rand() < 0.5) ? -1. : 1. ;*/
 	  mean += random[i];
 	}
       mean /= *n;
 
       /* realization number l */
-      s0[l] = 0.0;
+      s0[l] = 0.;
       for (j = 0; j < *m; j++)
 	{
-	  process = 0.0;
+	  process = 0.;
 	  for (i = 0; i < *n; i++)
 	    process += (random[i] - mean) * influ[i + j * (*n)];
 
@@ -396,26 +396,35 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
   Free(Tm);
 }
 
-/// Utility function for evtestA_derA
+/* FIXME ?? -- as the integrand uses indicators and hence is discontinuous, the numerical integrator gives tons
+   of 'ier=1' accuracy problem errors.   Rdqags() source (ier=1) mentions that one really should split the interval
+   into piecewise continuous sub-intervals one self !!
+ */
+
+// The integrand [called from vec_intgr() from evtestA_derA()] :
 double intgr(double x, double termUt, double termVt, double powUt, double powVt,
 	     double U, double V, double t, double n)
 {
-  double x1t = R_pow(x,1-t), xt = R_pow(x,t);
-  double indUt = (U <= x1t) - (int)(x1t * (n+1)) / n;
-  double indVt = (V <= xt) - (int)(xt * (n+1)) / n;
-  double xlogx = x * log(x), res = 0.0;
-  if (indUt) res +=  termUt * R_pow(x,powUt) * indUt / xlogx;
-  if (indVt) res += termVt * R_pow(x,powVt) * indVt / xlogx;
+  double
+      x1t = R_pow(x,1-t),  indUt = (U <= x1t) - (int)(x1t * (n+1)) / n,
+      xt  = R_pow(x, t),   indVt = (V <= xt ) - (int)(xt  * (n+1)) / n,
+      res = 0.;
+  if(indUt || indVt) {
+      double xlogx = x * log(x);
+      if (indUt) res += termUt * R_pow(x,powUt) * indUt / xlogx;
+      if (indVt) res += termVt * R_pow(x,powVt) * indVt / xlogx;
+  }
   return res;
 }
 
-/// Utility function for evtestA_derA
+
+// Vectorized intgr(), called from numerical integrator Rdqags(), from evtestA_derA
 void vec_intgr(double *x, int n, void *ex)
 {
-  int i;
-  double *arg = ex;
-  for (i = 0; i < n; i++)
-    x[i] = intgr(x[i], arg[0], arg[1], arg[2], arg[3],
+  double *arg = (double *) ex;
+  for (int i = 0; i < n; i++)
+    x[i] = intgr(x[i],
+		 arg[0], arg[1], arg[2], arg[3],
 		 arg[4], arg[5], arg[6], arg[7]);
   return;
 }
@@ -430,128 +439,131 @@ void vec_intgr(double *x, int n, void *ex)
  * @param u grid
  * @param v grid
  * @param m grid size
- * @param CFG if > 0, then An=CFG, otherwise Pickands
+ * @param CFG if != 0, then An=CFG, otherwise Pickands
+ * @param trace_lev: if > 0, print "progress" output to console
  * @param N number of multiplier replications
- * @param s0 N replications of the test statistic
+ * @param s0 ["Output"]: N replications of the test statistic
  * @author Ivan Kojadinovic
  */
-void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
-	     int *m, int *CFG, int *N, double *s0)
+void evtestA_derA(double *U, double *V, int *n,
+		  double *u, double *v, int *m, int *CFG_tr__etc,
+		  int *N, double *s0)
 {
-  double *influ = Calloc((*n) * (*m), double);
-  double *influ2 = Calloc((*n) * (*m), double);
-
-  double *S = Calloc(*n, double);
-  double *T = Calloc(*n, double);
-
+    int CFG       = CFG_tr__etc[0],
+	trace_lev = CFG_tr__etc[1],
+	report_err= CFG_tr__etc[2];
+  double  *influ  = Calloc((*n) * (*m), double), *influ2;
+  if(CFG) influ2 = Calloc((*n) * (*m), double);
   double *random = Calloc(*n, double);
 
-  /* for numerical integration: begin */
+  /* for numerical integration [Rdqags() <==> R's integrate()]: begin */
   double result, abserr;
   int last, neval, ier;
-  int limit=100;
   double reltol=0.0001;
   double abstol=0.0001;
-  int lenw = 4 * limit;
-  int *iwork = Calloc(limit, int);
-  double *work = Calloc(lenw, double);
-  double ex[8], lower = 0.0, upper = 1.0;
-  /* lower = 1.0 / (*n + 1), upper = *n / (*n + 1.0); */
+  int limit=100;         int   *iwork = Calloc(limit, int);
+  int lenw = 4 * limit;  double *work = Calloc(lenw,  double);
+  double ex[8],
+      lower = 0., // = 1. / (*n + 1.)
+      upper = 1.; // = *n / (*n + 1.)
+  int nErr = 0, errCnts[6] = {0,0, 0,0, 0,0};
   /* for numerical integration: end */
 
-  double pu, pv, d1, d2,  process,
-    mean, invsqrtn = 1.0 / sqrt(*n),
-    A, cA0, cA1, t, tp, tm, uv, loguv, loguvA,
-    Aterm, dAt, Ap, Am, termUt, termVt, powUt, powVt;
-
-  int i, j, l;
-
-  /* temporary arrays */
-  for (i = 0; i < *n; i++)
+  double // temporary arrays  S[], T[] :
+      *S = Calloc(*n, double),
+      *T = Calloc(*n, double);
+  for (int i = 0; i < *n; i++)
     {
       S[i] = - log(U[i]);
       T[i] = - log(V[i]);
     }
 
   /* correction terms */
-  if (*CFG)
+  double cA0, cA1;
+  if (CFG)
     {
-      cA0 = biv_logACFG(*n, S, T, 0.0);
-      cA1 = biv_logACFG(*n, S, T, 1.0);
+      cA0 = biv_logACFG(*n, S, T, 0.);
+      cA1 = biv_logACFG(*n, S, T, 1.);
     }
   else
     {
-      cA0 = biv_invAP(*n, S, T, 0.0);
-      cA1 = biv_invAP(*n, S, T, 1.0);
+      cA0 = biv_invAP(*n, S, T, 0.);
+      cA1 = biv_invAP(*n, S, T, 1.);
     }
 
   /* for each point of the grid */
-  for (j = 0; j < *m; j++)
-    {
-      uv = u[j] * v[j];
-      loguv = log(uv);
+  for (int j = 0; j < *m; j++) {
+      double
+	  uv = u[j] * v[j],
+	  loguv = log(uv),
+	  pu = loguv / log(u[j]),
+	  pv = loguv / log(v[j]);
 
-      pu = loguv / log(u[j]);
-      pv = loguv / log(v[j]);
+      if(trace_lev)
+	  REprintf(trace_lev >= 2 ? "\n>-> j=%d: pv=%g ...\n" : "j=%d, pv=%11g.\n",
+		   j, pv);
+      double
+	  t = 1. / pv,
+	  invsqrtn = 1. / sqrt(*n),
+	  tp = t + invsqrtn,
+	  tm = t - invsqrtn;
 
-      t = 1.0 / pv;
-      tp = t + invsqrtn;
-      tm = t - invsqrtn;
-
-      if (*CFG)
+      double A, loguvA, Aterm, Ap, Am, dAt, termUt, termVt, powUt, powVt;
+      if (CFG)
 	{
 	  A = exp(biv_logACFG(*n, S, T, t)
-		  - (1.0 - t) * cA0 - t * cA1);
+		  - (1. - t) * cA0 - t * cA1);
 	  loguvA = loguv * A;
 	  Aterm = exp(loguvA) * loguvA;
 
 	  Ap = exp(biv_logACFG(*n, S, T, tp)
-		   - (1.0 - tp) * cA0 - tp * cA1);
+		   - (1. - tp) * cA0 - tp * cA1);
 	  Am = exp(biv_logACFG(*n, S, T, tm)
-		   - (1.0 - tm) * cA0 - tm * cA1);
-	  dAt = (Ap - Am) / (2.0 * invsqrtn);
+		   - (1. - tm) * cA0 - tm * cA1);
+	  dAt = (Ap - Am) / (2. * invsqrtn);
 
 	  termUt = A - t * dAt;
-	  termVt = A + (1.0 - t) * dAt;
-	  powUt = A + t - 1.0;
+	  termVt = A + (1. - t) * dAt;
+	  powUt = A + t - 1.;
 	  powVt = A - t;
 	}
       else
 	{
-	  A = 1.0 / (biv_invAP(*n, S, T, t)
-		     - (1.0 - t) * (cA0  - 1.0)
-		     - t * (cA1 - 1.0));
+	  A = 1. / (biv_invAP(*n, S, T, t)
+		     - (1. - t) * (cA0  - 1.)
+		     - t * (cA1 - 1.));
 	  loguvA = loguv * A;
 	  Aterm = exp(loguvA) * loguvA * A;
 
-	  Ap = 1.0 / (biv_invAP(*n, S, T, tp)
-		       - (1.0 - tp) * (cA0  - 1.0)
-		       - tp * (cA1 - 1.0));
-	  Am = 1.0 / (biv_invAP(*n, S, T, tm)
-		       - (1.0 - tm) * (cA0  - 1.0)
-		       - tm * (cA1 - 1.0));
-	  dAt = (Ap - Am) / (2.0 * invsqrtn);
+	  Ap = 1. / (biv_invAP(*n, S, T, tp)
+		       - (1. - tp) * (cA0  - 1.)
+		       - tp * (cA1 - 1.));
+	  Am = 1. / (biv_invAP(*n, S, T, tm)
+		       - (1. - tm) * (cA0  - 1.)
+		       - tm * (cA1 - 1.));
+	  dAt = (Ap - Am) / (2. * invsqrtn);
 
-	  termUt = (A - t * dAt) / (A + t - 1.0);
-	  termVt = (A + (1.0 - t) * dAt) / (A - t);
-	  powUt = (A + t - 1.0) / (1.0 - t);
+	  termUt = (A - t * dAt) / (A + t - 1.);
+	  termVt = (A + (1. - t) * dAt) / (A - t);
+	  powUt = (A + t - 1.) / (1. - t);
 	  powVt = (A - t) / t;
 	}
 
-      /*d1 = (A - t * dAt) * R_pow(uv, A - 1.0 + t);
-	d2 = (A + (1.0 - t) * dAt) * R_pow(uv, A - t);*/
+     R_CheckUserInterrupt();
+      /*d1 = (A  -    t   * dAt) * R_pow(uv, A - 1 + t);
+	d2 = (A + (1 - t) * dAt) * R_pow(uv, A - t);*/
+      double
+	  d1 = der1bivCn(U, V, *n, u[j], v[j]),
+	  d2 = der2bivCn(U, V, *n, u[j], v[j]);
 
-      d1 = der1bivCn(U, V, *n, u[j], v[j]);
-      d2 = der2bivCn(U, V, *n, u[j], v[j]);
-
-      for (i = 0; i < *n; i++) /* for each pseudo-obs */
-	{
-	  if (*CFG)
-	    {
-	      ex[0] = termUt; ex[1] = termVt;
-	      ex[2] = powUt; ex[3] = powVt;
-	      ex[4] = U[i]; ex[5] = V[i];
-	      ex[6] = t; ex[7] = *n;
+      for (int i = 0; i < *n; i++) { /* for each pseudo-obs */
+	  if (CFG) {
+	      if(i == 0) {
+		  ex[0] = termUt; ex[1] = termVt;
+		  ex[2] = powUt;  ex[3] = powVt;
+		  ex[6] = t;      ex[7] = *n;
+	      }
+	      ex[4] = U[i];   ex[5] = V[i];
 
 	      Rdqags(vec_intgr, (void *)ex, &lower, &upper,
 		     &abstol,  &reltol,
@@ -559,62 +571,82 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
 		     &limit,  &lenw, &last,
 		     iwork, work);
 
-	      /* if (ier)
-		 Rprintf("%lf %lf %d\n", result, abserr, ier); */
+	      if(ier && report_err) {
+		  // ier=1  happens a lot (99%) -- because integrand has jumps
+		  /* if(ier > 1) */
+		  /*     REprintf(" i=%d: Rdqags()=%lf, abserr=%lf, ier=%d *error*\n", i, */
+		  /* 	       result, abserr, ier); */
+		  nErr++;
+		  errCnts[ier-1]++;
+	      }
+	      if(trace_lev >= 2)
+		  REprintf(" i=%d: Rdqags()=%10g, abserr=%6g, neval=%d, last=%d\n", i,
+			   result, abserr, neval, last);
 
 	      influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
-		- d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
-		+ Aterm * log(MIN(pu * S[i], pv * T[i]));
-	      influ2[i + j * (*n)] =  Aterm * result;
-	    }
-	  else
-	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
-	      - d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
-	      + Aterm * (MIN(pu * S[i], pv * T[i])
-			 - termUt * (1.0 - R_pow(U[i],powUt))
-			 - termVt * (1.0 - R_pow(V[i],powVt)));
-
-	  influ[i + j * (*n)] *= invsqrtn;
-	  influ2[i + j * (*n)] *=  invsqrtn;
-	}
-    }
+		  - d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
+		  + Aterm * log(MIN(pu * S[i], pv * T[i]));
+	      influ2[i + j * (*n)] =  Aterm * result * invsqrtn;
+	  }
+	  else {
+	      influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
+		  - d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
+		  + Aterm * (MIN(pu * S[i], pv * T[i])
+			     - termUt * (1. - R_pow(U[i],powUt))
+			     - termVt * (1. - R_pow(V[i],powVt)));
+	  }
+	  influ [i + j * (*n)] *= invsqrtn;
+      } // for i
 
+    } // for j
+  if(trace_lev)
+      REprintf("\n---> completed influ[%d,%d] integration computations\n", *n, *m);
+  if(nErr && report_err) {
+      #define REPRINT6(IEX)				\
+        for(int i=0; i < 6; i++) REprintf(" %3d", IEX);	\
+        REprintf("\n")
+      REprintf("Had %d integration errors (= %4.1f%%) with the following counts of each ier=1,..,6:\n",
+	       nErr, 100. * nErr / ((double) *n * *m) );
+      REPRINT6(   i+1    );
+      REPRINT6(errCnts[i]);
+  }
   GetRNGstate();
 
   /* generate N approximate realizations */
-  for (l = 0; l < *N; l++)
-    {
+  if(trace_lev >= 2) REprintf("Random Realizations:\n");
+  for (int l = 0; l < *N; l++) {
       /* generate n variates */
-      mean = 0.0;
-      for (i=0;i<*n;i++)
-	{
-	  random[i] = norm_rand(); /*(unif_rand() < 0.5) ? -1.0 : 1.0 ;*/
+      double mean = 0.;
+      for (int i=0; i < *n; i++) {
+	  random[i] = norm_rand(); /*(unif_rand() < 0.5) ? -1. : 1. ;*/
 	  mean += random[i];
-	}
+      }
       mean /= *n;
-
+      if(trace_lev >= 2) {
+	  if(l % 50 == 1) REprintf(" l=%d\n", l); else REprintf(".");
+      }
       /* realization number l */
-      s0[l] = 0.0;
-      for (j = 0; j < *m; j++)
-	{
-	  process = 0.0;
-	  for (i = 0; i < *n; i++)
-	    if (*CFG)
-	      process += (random[i] - mean) * influ[i + j * (*n)]
-		+ random[i] * influ2[i + j * (*n)];
-	    else
-	      process += (random[i] - mean) * influ[i + j * (*n)];
+      s0[l] = 0.;
+      for (int j = 0; j < *m; j++) {
+	  double process = 0.;
+	  for (int i = 0; i < *n; i++)
+	      if (CFG)
+		  process += (random[i] - mean) * influ[i + j * (*n)]
+		      + random[i] * influ2[i + j * (*n)];
+	      else
+		  process += (random[i] - mean) * influ[i + j * (*n)];
 
 	  s0[l] += process * process;
-	}
+      }
       s0[l] /= *m;
-    }
-
+  }
+  if(trace_lev)
+      REprintf("\n---> Finished N=%d random realizations s0[]\n", *N);
   PutRNGstate();
 
 
   Free(influ);
-  Free(influ2);
+  if(CFG) Free(influ2);
   Free(random);
 
   Free(S);
@@ -646,7 +678,7 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
 		  int *CFG, double *stat, double *offset)
 {
   int i, j;
-  double s = 0.0, diff, cA0, cA1, Aj, t, loguv;
+  double s = 0., diff, cA0, cA1, Aj, t, loguv;
   double *S = Calloc(*n, double);
   double *T = Calloc(*n, double);
 
@@ -659,13 +691,13 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
   /* correction terms */
   if (*CFG)
     {
-      cA0 = biv_logACFG(*n, S, T, 0.0);
-      cA1 = biv_logACFG(*n, S, T, 1.0);
+      cA0 = biv_logACFG(*n, S, T, 0.);
+      cA1 = biv_logACFG(*n, S, T, 1.);
     }
   else
     {
-      cA0 = biv_invAP(*n, S, T, 0.0);
-      cA1 = biv_invAP(*n, S, T, 1.0);
+      cA0 = biv_invAP(*n, S, T, 0.);
+      cA1 = biv_invAP(*n, S, T, 1.);
     }
 
   for (j = 0; j < *m; j++)
@@ -675,18 +707,17 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
 
       if (*CFG)
 	Aj = exp(biv_logACFG(*n, S, T, t)
-		 - (1.0 - t) * cA0 - t * cA1);
+		 - (1. - t) * cA0 - t * cA1);
       else
-	Aj = 1.0 / (biv_invAP(*n, S, T, t)
-		    - (1.0 - t) * (cA0  - 1.0)
-		    - t * (cA1 - 1.0));
-      if (*offset < 0.0)
+	Aj = 1. / (biv_invAP(*n, S, T, t)
+		    - (1. - t) * (cA0  - 1.)
+		    - t * (cA1 - 1.));
+      if (*offset < 0.)
 	diff = bivCn(U, V, *n, u[j], v[j]) - exp(loguv * Aj);
       else
 	diff = bivCn(U, V, *n, u[j], v[j]) * (*n) / (*n+1)
 	  + (*offset)/(*n+1) - exp(loguv * Aj);
 
-
       s += diff * diff;
     }
 
@@ -695,5 +726,3 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
   Free(S);
   Free(T);
 }
-
-
